package Amethyst::Brain::Infobot::Module::Statement;

use strict;
use vars qw(@ISA $RE_VERB $RE_QUESTION);
use Amethyst::Message;
use Amethyst::Store;
use Amethyst::Brain::Infobot;
use Amethyst::Brain::Infobot::Module;

@ISA = qw(Amethyst::Brain::Infobot::Module);
$RE_VERB = "is|are|has";
$RE_QUESTION = "what|who|where|which|wot|wtf";

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(
					Name		=> 'Statement',
					# Regex		=> qr/(?:karma|\+\+|--)/i,
					Usage		=> 'Make a statement. Ask a question.',
					Description	=> "Statement handler",
					@_
						);

	die "No FactoidWrite store in Infobot config"
					unless $self->{Infobot}->{FactoidWrite};

	$self->{WriteStore} = new Amethyst::Store(
					Source	=> $self->{Infobot}->{FactoidWrite},
						);

	my @stores = ($self->{WriteStore});
	foreach (@{ $self->{Infobot}->{FactoidRead} }) {
		my $store = new Amethyst::Store(
					Source	=> $_,
						);
		push(@stores, $store);
	}

	$self->{Stores} = \@stores;

	return bless $self, $class;
}

sub forget {
	my ($self, $message, $skey) = @_;

	$skey = lc $skey;
	$skey =~ s/[.!?]+$//;

	my $store = $self->{WriteStore};

	if ($store->get($skey)) {
		$store->unset($skey);
		my $reply = $self->reply_to($message,
						"I forgot $skey.");
		$reply->send;
	}
	else {
		foreach my $store (@{ $self->{Stores} }) {
			if ($store->get($skey)) {
				my $reply = $self->reply_to($message,
								"I have been instructed not to " .
								"forget $skey. " .
								"It is in readonly store " .
								"$store->{Source}.");
				$reply->send;
			}
			return undef;
		}

		my $reply = $self->reply_to($message,
						"I didn't know about $skey.");
		$reply->send;
	}

	return undef;
}

sub question {
	my ($self, $message, $skey) = @_;

	foreach my $store (@{ $self->{Stores} }) {

		$skey = lc $skey;
		$skey =~ s/[.!?]+$//;
		my $data = $store->get($skey);
		foreach my $sval (keys %$data) {
			my $href = $data->{$sval};
			my $key = $href->{key} || $skey;
			my $val = $href->{val} || $sval;

			my $msg = "$key $href->{verb} $val";

			my $reply = $self->reply_to($message, ucfirst $msg);
			$reply->send;
		}
	}
}

sub statement {
	my ($self, $message, $key, $verb, $val) = @_;

	my $store = $self->{WriteStore};
	my $skey = lc $key;
	my $sval = lc $val;

	my $data = $store->get($skey);
	$data->{$sval} = {
		verb	=> $verb,
		user	=> $message->user,
			};
	$data->{$sval}->{key} = $key if $key ne $skey;
	$data->{$sval}->{val} = $val if $val ne $sval;

	$store->set($skey, $data);
}

sub process {
    my ($self, $message) = @_;

	my $content = $message->content;
	my $who = $message->user;

	$content =~ s/\s+/ /g;

	$content =~ s/\bi am /$1$who is /i; 
	$content =~ s/\bmy /$1$who\'s /ig;
	$content =~ s/\bour /$1$who\'s /ig;
	# $content =~ s/\byour /$1$name\'s /ig;

	$content =~ s/\bisn't /is not /ig;
	$content =~ s/\baren't /are not /ig;

	# if ($content =~ /^forget\s+(?:(?:a|an|the)\s+)?(.*)/i) {
	if ($content =~ /^forget\s+(.*)/i) {
		return $self->forget($message, $1);
	}

	# Modification ignored for now
	if ($content =~ /^($RE_QUESTION)\s+($RE_VERB)\s+(.*)/io) {
		return $self->question($message, $3);
	}
	elsif ($content =~ /^(.*?)\s*\?/) {
		return $self->question($message, $1);
	}

	foreach (split(/(?:[?!:;]|\.\s)/, $content)) {
		if (/^(.*?)\s+($RE_VERB)\s+(.*)/io) {
			my ($key, $verb, $val) = ($1, lc $2, $3);
			# print STDERR "Considering $_\n";

			$key =~ s/^.*\s+that\s+//;		# $N verbs that X is Y
			$key =~ s/^.*\s+notes?\s+//;	# $N notes X is Y
			$key =~ s/^.*,//;
			$key =~ s/\s+/ /g;
			$key =~ s/^\s*//;
			$key =~ s/\s*$//;

			next if length $key > 20;
			next unless $key =~ /\S/;
			next if $key =~ /\bwhich\b/;
			next if $key =~
				/^(?:if|it|that|they|there|so|you|$RE_QUESTION)$/io;

			print STDERR "Creating statement from '$key'\n";

			$self->statement($message, $key, $verb, $val);
		}
	}

	# forget X
	# X is Y
	# I/you/we am/is/are Y
	# who/what/which/where am/is/are Y
}

1;
